
ArrayList brushes;

float current_speed = 6.0;
float n_ary = 5;
float current_da = 0.0;
int current_rad = 10;
float init_a = 0;

int r = 250;
int g = 250;
int b = 250;

color current_colour = color(r,g,b);
color bg_colour = color(0,0,0);

int current_pic_no = 0;
boolean stroking = true;
boolean wrapping = true;


void setup() {
  size(600,600);
  smooth();
  brushes = new ArrayList();
  background(0); 
 
}

void draw() {
  for (int i=0;i<brushes.size();i++) {
      ((Brush)brushes.get(i)).look_ahead();
  }
  for (int i=0;i<brushes.size();i++) {
      ((Brush)brushes.get(i)).step();
  }
}


void save_pic() {
  save("flower_brush"+current_pic_no+".jpg");
  current_pic_no=current_pic_no+1;
}

void clear_screen() {
    bg_colour = current_colour;
    background(bg_colour); 
}

void clear_screen(color c) {
    background(c);
}

void mousePressed() {
  for (float a=init_a; a<init_a+(2*PI); a=a+2*PI/n_ary) {  
    brushes.add(new Brush(mouseX, mouseY, a, current_speed, current_colour));
  }
}

void toggle_stroke() {
  if (stroking) {
    noStroke();
  } else {
    stroke(bg_colour);
  }
  stroking = ! stroking;
  
}

void keyPressed() {
  switch(key) {
    case 'q' : 
      r = 250;
      break; 
    case 'a' : 
      r = 150;
      break; 
    case 'z' : 
      r = 50;
      break; 
      
    case 'w' : 
      g = 250;
      break;
    case 's' : 
      g = 150;
      break;
    case 'x' : 
      g = 50;
      break;
      
    case 'e' : 
      b = 250;
      break;
    case 'd' : 
      b = 150;
      break;
    case 'c' : 
      b = 50;
      break;
      
    case '2' :
      n_ary = 2;
      break;
    case '3' :
      n_ary = 3;
      break;
    case '4' :
      n_ary = 4;
      break;
    case '5' :
      n_ary = 5;
      break;
    case '6' :
      n_ary = 6;
      break;
    case '7' :
      n_ary = 7;
      break;
    case '8' :
      n_ary = 8;
      break;
      
    case '/' :
      save_pic();
      break;
      
    case ' ' :
      clear_screen();
      break;

    case 'n' :
      clear_screen(color(0,0,0));
      break;
    
    case '.' :
      current_da = current_da + PI/40;
      break;
      
    case ',' :
      current_da = current_da - PI/40;
      break;

    case '[' :
      init_a = init_a + PI/4;
      break;
      
    case ']' :
      init_a = init_a - PI/4;
      break;

    
    case 't' :
      current_rad = 20;
      break;
    case 'g' :
      current_rad = 10;
      break;
    case 'b' :
      current_rad = 5;
      break;    
    
    case '#' :
      toggle_stroke();
      break;
      
    case '=' :
      wrapping = !wrapping;
      break;
      
  }
  
  current_colour = color(r,g,b);
}
class Brush {
   int x;
   int y; 
   int dx; // x velocity
   int dy; // y velocity
   
   float d; // speed
   
   float a; // angle
   float da; // angular velocity

   int counter;
   
   int radius;
   color colour;
   color in_front;
 
   boolean go_forward_flag; // go used for in_front_test to signal its ok to go forward  

   void setup(int x, int y, float a, float d, color colour) {
      this.x = x;
      this.y = y;
      this.d = d;

      this.set_angle(a);
      
      this.da = current_da;
      this.colour = colour;
   
      this.radius = current_rad; 
      this.counter = 20;
      this.go_forward_flag=true;
   }

   void set_angle(float a) {
      this.a = a;
      this.dx = (int)(this.d * cos(a));
      this.dy = (int)(this.d * sin(a));
   }
   
   Brush() {
      this.setup(0,0,0.0,0.0,color(0,0,0));
   }
   
   Brush(int x, int y, float a, float d, color colour) {
      this.setup(x,y,a,d,colour);     
   }
   
   boolean not_black(color c) {
     return ( (red(c) != red(bg_colour)) || 
              (green(c) != green(bg_colour)) || 
              (blue(c) != blue(bg_colour)) );
   }

   PVector wrap(int x, int y) {
     if (wrapping) {
       return new PVector((width + x) % width, (height + y) % height);
     } else {
       return new PVector(x,y);
     }
   }
   
   void look_ahead() {
     PVector look = wrap(this.x+(this.dx*2),this.y+(this.dy*2));
     this.in_front = get((int)look.x,(int)look.y);
   }
   
   void auto_destruct() {
     brushes.remove(brushes.indexOf(this));
   }
 
   void expiry_test() {
     this.counter = this.counter - 1;
     if (this.counter < 0) {        
       this.auto_destruct();
     }
   }
   
   void in_front_test() {
     if (this.not_black(this.in_front)) {
       this.go_forward_flag = false;
       //this.auto_destruct();  
     }
   }
   
   void step() {
     if (go_forward_flag) {
       PVector next = wrap(this.x+this.dx,this.y+this.dy);
       this.x = (int)next.x;
       this.y = (int)next.y;

       fill(this.colour);
       ellipse(this.x,this.y,this.radius,this.radius);  
     }
     
     this.set_angle(this.a+this.da);
     
     this.expiry_test();
     this.in_front_test();
   
   }
}


class Turner extends Brush {
   Turner(int x, int y, float a, float d, color colour) {
      this.setup(x,y,a,d,colour);     
   }

   void in_front_test() {
     if (this.not_black(this.in_front)) {
       this.set_angle(this.a + PI/6);
       this.go_forward_flag = false;
     } else {
       this.go_forward_flag = true;
     }
   }

}

